;========================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;========================================
;	SOUND FOR THE ZX81
;    R. Bruneaus, Sept. 1994
;    Sinc-Link Vol. 12 No. 4
;========================================

ORG 4084

@CHAdd=4016
@LastK=4025
@Seed=4032
@Frames=4034
@Scanning=0F55
@FPtoA=15CD
@FindInt=0EA7
@PRBUFF00=403C
@PRBUFF01=403D
@PRBUFF02=403E
@PRBUFF03=403F
@PRBUFF04=4040
@PRBUFF05=4041
@PRBUFF06=4042
@PRBUFF07=4043

;==========================
;Comma check
;==========================
@Comma
	PUSH AF 
	LD A,(HL) 
	CP 1A			;Comma?
	JR Z CommaOK		;IF NOT Comma THEN ErrorP
	RST 08			;Error: not a comma seperator
	18			;Error Code'P'
@CommaOK
	POP AF 
	RET
;==========================
;Set Reg A to parameter value (0 to 256)
;==========================
@SetA
	LD (CHAdd),HL		;Start of parameter
	CALL Scanning		;Call the expression evaluator
	PUSH HL 		;PUSH BASIC line addr
	CALL FPtoA		;A=Results
	POP HL 			;POP BASIC line addr
	RET
;==========================
;Set Reg BC to parameter value (0 to 65535)
;==========================
@SetBC
	LD (CHAdd),HL		;Start of parameter
	CALL Scanning		;Call the expression evaluator
	PUSH HL 		;PUSH BASIC line addr
	CALL FindInt		;BC=Results
	POP HL 			;POP BASIC line addr
	RET
;==========================
;Get small value parameter
;==========================
@GetParam
	CALL SetA
	CALL Comma
	RET
;==========================
;Get large value parameter
;==========================
@Int2BC
	CALL SetBC		
	CALL Comma
	RET
;==========================
;Check for semicolon
;==========================
@SemiColon
	LD HL,(CHAdd)		;BASIC line addr
	LD A,(HL) 		;Check for semicolon
	CP 19			;Semicolon?
	JR Z NextCom		;IF Semicolon THEN process next command
	CP 76 			;N/L?
	JR Z EOL		;IF NOT N/L THEN ErrorE
@ErrorE
	RST 08
	0D			;Error Code'E'
@EOL				;IF EOL THEN RETURN to BASIC
	POP HL 			;Restore CHAdd to BASIC line addr
	LD (CHAdd),HL 		
	LD BC,0000 		;Return 0 to BASIC
	RET
;==========================
;Parse BASIC line and do commands
;==========================
@Start
	LD HL,(CHAdd)		;BASIC line addr
	LD A,(HL)
	CP DE 			;"THEN"? 
	JR Z Then		;IF NOT "THEN" THEN ErrorI
@ErrorI			
	RST 08			;"I"nvalid statement
	11			;Error Code'I'
@Then
	PUSH HL 		;Save BASIC line addr
	INC HL			;Next BASIC chr
	LD A,(HL)		;A=Command character 
	LD DE, EOL		;PUSH return to BASIC routine
	PUSH DE 		
	CP F5			;"PRINT"? 
	JR NZ ErrorI		;IF NOT "LPRINT" THEN ErrorI
	POP DE			;POP return to BASIC routine
@NextCom
	LD DE,SemiColon 	;Line continue routine
	PUSH DE			;PUSH line continue routine
	INC HL			;Next BASIC chr 
	LD A,(HL)
	LD B,A 			;Save command character
@LineLoop
	INC HL			;Next BASIC chr addr
	LD A,(HL) 		;Next BASIC chr
	CP 76	 		;IF EOL THEN ErrorI, Parameter missing
	JR Z ErrorI
	CP 19			;Semicolon?
	JR NZ LineLoop		;Loop until EOL or ";" found
	LD A,B 			;A=Command character
	CP "M"			;"M"usic command
	JP Z Music
	CP "S"			;"S"ound command
	JP Z Sound
	CP "P"			;"P"lay command 
	JP Z Play
	CP "W"			;"W"ait command 
	JP Z Pause
	JR ErrorI		;IF NOT valid command THEN ErrorI
;==========================
;MUSIC command
;==========================
@Music
	INC HL 			;Start of MUSIC parameters section
	CALL GetParam		;Get volume value
	CP 10			
	JR NC ErrorM		;IF Volume>16 THEN ErrorM
	AND A 
	JR NZ NotZero1		;Volume 0?
	ADD A,10		;IF Volume=0 THEN use envelope control set by PLAY
@NotZero1
	LD (PRBUFF01),A		;Store volume
	INC HL			;Next parameter
	CALL GetParam		;Get NOTE value
	CP 0D 			
	JR NC ErrorM		;IF NOTE>13 THEN ErrorM
	AND A 
	JR Z ErrorM		;IF NOTE=0 THEN ErrorM
	SUB 01			;A=0 TO 12
	ADD A,A 		;A=0,2,4,6,...22
	PUSH HL 		;PUSH BASIC line addr
	LD HL,Notes		;Note table base addr
	LD B,00 		;Locate NOTE data in table
	LD C,A
	ADD HL,BC 
	LD A,(HL)
	INC HL 
	LD B,(HL)
	LD C,A
	LD (PRBUFF03),BC	;Fine tone value
	POP HL			;POP BASIC line addr 
	INC HL
	CALL GetParam		;Get OCTAVE value
	CP 07			;IF OCTAVE>7 THEN ErrorM 
	JR NC ErrorM
	INC A 			;OCTAVE=OCTAVE+1
	LD BC,(PRBUFF04) 	;Adjust Coarse & Fine tone
	LD B,A 
	LD A,(PRBUFF03) 
	JR Octave1
@Octave2
	RR C 
	RES 7,C 
	RR A 
	AND A 
@Octave1
	DJNZ Octave2
	LD (PRBUFF03),A		;Fine tone
	LD A,C 
	LD (PRBUFF05),A		;Coarse tone
	INC HL			;Get channel parameter
	CALL SetA
	CP 04 			;IF Channel>4 THEN ErrorM
	JR NC ErrorM
	AND A 
	JR Z ErrorM 		;IF Channel=0 THEN ErrorM
	PUSH AF 
	ADD A,07 
	LD (PRBUFF00),A		;R7-Channel enable
	POP AF 
	SUB 01 
	ADD A,A 
	LD (PRBUFF02),A		;Fine tone
	INC A 
	LD (PRBUFF04),A		;R1, Coarse tone
	PUSH HL 
	LD B,03 		;3 parameters to process
	CALL SendData
	JP SemiColon
@ErrorM
	RST 08			;"M"USIC error
        15 			;Error Code'M'
;==========================
;SOUND command
;==========================
@Sound
	INC HL 			;Start of SOUND parameters section
	CALL GetParam		;Get volume value
	CP 10			
	JR NC ErrorS		;IF Volume>15 THEN ErrorS
	AND A 
	JR NZ NotZero2		;Volume 0?
	ADD A,10		;IF Volume=0 THEN use envelope control set by PLAY
@NotZero2
	LD (PRBUFF01),A		;Store volume
	INC HL			;Next parameter
	CALL Int2BC		;Get pitch value
	LD A,C 			;Store pitch value
	LD (PRBUFF03),A		;Fine tone value
	LD A,B 
	LD (PRBUFF05),A		;Coarse tone value
	INC HL 			;Next parameter
	CALL SetA		;Get channel value
	CP 07
	JR NC ErrorS		;IF channel>6 THEN ErrorS
	AND A
	JR Z ErrorS		;IF channel=0 THEN ErrorS
	PUSH AF 		;PUSH channel value
	CP 04 			
	JR NC Channel456	;IF channel=4,5,or 6 THEN GOTO Channel456
	ADD A,07		;A=8,9 or 10
	LD (PRBUFF00),A		;R8, R9 or R10 (A,B,C Volume)
	POP AF 			;POP channel value, 1,2,3
	SUB 01 			;A=0,1,2
	ADD A,A 		;A=0,2,4
	LD (PRBUFF02),A		;R0, R2 or R4 (A,B,C fine tone)
	INC A 			;A=1,3,5
	LD (PRBUFF04),A		;R1, R3 or R5 (A,B,C coarse tone)
	JR SoundSend		;Send data
@ErrorS
	RST 08			;"S"ound error
	1B			;Error Code'S'
;==========================
;For channels 4,5,6 use R6-Noise Period
;==========================
@Channel456

	ADD A,04 		;A=8,9,10
	LD (PRBUFF00),A		;R8, R9 or R10 (A,B,C Volume)
	POP AF			;POP channel value 
	LD A,06			;R6
	LD (PRBUFF02),A		;R6 Noise period
	LD (PRBUFF04),A		;R6 Noise period
	LD A,(PRBUFF03)		;C6 value
	CP 20 			;32
	JP NC ErrorS		;IF C6 value>31 THEN ErrorS
	AND A 
	JR Z ErrorS		;IF C6 value=0 THEN ErrorS
	LD (PRBUFF05),A		;R6 value
@SoundSend
	PUSH HL 		;Save BASIC line addr
	LD B,03 		;Three parameters to send
	CALL SendData		;Send data
	JP SemiColon		;Do next command if any
;==========================
;PLAY command
;==========================
@Play
	INC HL			;Start of parameters 
	CALL Int2BC		;Get duration value
	LD A,C			;C11
	LD (PRBUFF01),A
	LD A,B 			;C12
	LD (PRBUFF03),A
	LD A,0B 		;R11
	LD (PRBUFF00),A
	INC A 			;R12
	LD (PRBUFF02),A
	INC HL 
	CALL GetParam		;Get envelope value
	CP 08 
	JR NC ErrorP		;IF A>7 THEN ErrorP
	AND A 			;A=0?
	JR Z EnvSet		;A=Envelope 0
	CP 01			;A=1?
	JR NZ Play2
	LD A,06			;A=Envelope 6
	JR EnvSet
@Play2
	CP 02			;A=2?
	JR NZ AddSeven
	DEC A			;A=1
@AddSeven
	ADD A,07 		;A=Envelope 8,10,11,12,13 or 14
@EnvSet
	LD (PRBUFF05),A		;C13 Envelope value
	LD A,0D 		;R13
	LD (PRBUFF04),A
	LD A,07 		;D7 Noise & tones
	LD (PRBUFF06),A
	INC HL 
	CALL GetParam		;Get noise value
	CP 08 
	JR NC ErrorP		;IF A>9 THEN ErrorP
	SLA A 
	SLA A 
	SLA A 			;A=A*8
	LD (PRBUFF07),A		;C7
	INC HL 
	CALL SetA		;Get tone value
	CP 08 
	JR NC ErrorP		;IF A>9 THEN ErrorP
	PUSH HL 		;PUSH BASIC line addr
	LD HL,PRBUFF07 
	OR (HL) 
	CPL 
	LD (PRBUFF07),A		;C7
	CP FF 			;All disabled?
	JR Z DisableABC		;IF FF THEN Diable A,B and C
	LD B,04 		;4 Parameters to send
	CALL SendData		;Send parameters to interface
	JP SemiColon		;Next command
@ErrorP
	RST 08	 		;"P"LAY error
	18 			;Error Code'P'
;=======================
;Shut off all sounds by clearing all registers.
;=======================
@DisableABC
	XOR A 			;Clear all Registers
	LD (PRBUFF00),A
	LD (PRBUFF01),A
	LD B,0E 		;14 Registers to set to 0
@Reset
	LD HL,PRBUFF00 		;(HL)=0,1,2,...14
	LD A,(HL) 		;R0,R1,R2,...R14
	OUT DF,A		;Sound Card Port
	INC (HL) 		;Increase register number
	INC HL 			;CX
	LD A,(HL) 		;CX=0
	OUT 1F,A 		;Sound card DATA
	DJNZ Reset		;Reset the registers
	LD A,07			;D7 Noise & tones 
	OUT DF,A		;Sound Card Port
	LD A,FF			;C7 disable A,B & C 
	OUT 1F,A		;Sound card DATA
	JP SemiColon		;Next command
;==========================
;Send data to AY
;==========================
@SendData			;Registers and data stored in PRBUFF area
	LD HL,PRBUFF00
	LD A,(HL) 		;A=DX number
	OUT DF,A		;Sound Card Port
	INC HL 			;CX pointer
	LD A,(HL) 		;CX value
	OUT 1F,A		;Sound card DATA
	INC HL 			;Next parameter
	DJNZ SendData+3		;Loop until all parameters sent
	RET
;==========================
;Wait XXXX
;Causes the program to wait for V/50 secs if in SLOW
;mode.  XXXX must be positive and less than 32768.
;Pressing any key aborts the pause.
;==========================
@Pause
	INC HL			;Get pause parameter
	CALL SetBC
	SET 7,B
	LD (Frames),BC		;Set frames 
@PLoop
	LD HL,(Frames)		;Loop until Frames reaches 0.
	LD A,H 			;Test for Frames=0
	AND 7F 
	OR L 
	RET Z			;RET to JP Semicolon
	LD A,(Lastk)		;Check LastK
	CP FF			;Key down?
	JR Z PLoop		;IF no key press THEN continue pause
	RET			;RET to JP Semicolon
;==========================
@Notes
	F1 06			;Note 01 C
	8E 06			;Note 02 C#
	30 06			;Note 03 D
	D7 05			;Note 04 D#
	83 05			;Note 05 E
	34 05			;Note 06 F
	E9 04			;Note 07 F#
	A2 04			;Note 08 G
	60 04			;Note 09 G#
	20 04			;Note 10 A
	E6 03			;Note 11 A#
	AE 03			;Note 12 B
