;========================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;========================================

ORG 42BE

@DFile=400C			;Start of D-FILE
@Seed=4032
@Swap=407B			;Swap characters
@MineCount=4021			;Mine count, 16417

@JumpTable
	JR Mines
	JR SwapDE
	JR Left
	JP Invert
	JP Explosion
	JP Enterprise
	00 00 00
	00 00 00
	00 00 00
	00 00 00
;=============================
;Print mine at random location
;=============================
@Mines
	LD HL,MineCount
	LD B,(HL)		;Mine count
@NextMine
	PUSH BC
@Redo
	CALL RNDNum
	LD HL,(DFILE)
	INC HL
	ADD HL,BC		;Add offset to DFile
@Occupied
	LD A,(HL)		;Check if location occupied
	CP 80
	JR NZ Redo
	LD (HL),97
	POP BC			;Mines counter
	DJNZ NextMine
	RET
;=============================
;Generate a random number between 0 and 32768
;This routine extracted from MCoder II
;=============================
@RNDNum
	LD DE,(SEED)
	LD H,E 
	LD L,FD 
	LD A,D 
	OR A 
	LD B,00 
	SBC HL,DE 
	SBC A,B 
	SBC HL,DE 
	SBC A,B 
	LD E,A 
	LD D,B 
	SBC HL,DE 
	JR NC SetSeed
	INC HL 
@SetSeed
	LD (SEED),HL
;=============================
;This section limits the range to 0 to 792
;=============================
	LD B,00		
	LD C,L
	LD A,H
@MSB3
	CP C0		;MSB>192 (49152)?
	JR C MSB2
	LD B,03
	LD A,C
	CP 18
	JR C LSBOK
	LD A,18		;Trim LSB
@LSBOK
	LD C,A
	RET
@MSB2
	CP 80		;MSB>128 (32768)?
	JR C MSB1
	LD B,02
	RET
@MSB1
	CP 40		;MSB>64?
	JR C MSB0
	LD B,01
@MSB0
	RET
;========================================
;Swap chr D with chr E
;========================================
@SwapDE
	LD DE,(Swap)	;Chrs to Swap
	LD BC,0319
	LD HL,(DFile)	;Get DFile
	LD A,D		;Chr to replace
@SLoop1
	CPI
	RET PO
	JR NZ SLoop1
	DEC HL
	LD (HL),E	;Replace chr
	INC HL
	JR SLoop1
;========================================
;Scroll left
;========================================
@Left
	LD HL,(DFile)
	LD DE,01BF 
	ADD HL,DE 
	LD B,08 
@LLoop1
	DEC HL 
	LD C,(HL) 
	LD (HL),80 
@LLoop2
	DEC HL 
	LD A,(HL) 
	CP 76 
	JR Z LLoop3
	JR LLoop4
@LLoop3
	DJNZ LLoop1
	LD C,00 
	RET
@LLoop4
	LD (HL),C 
	LD C,A 
	JR LLoop2

;========================================
;Invert screen
;========================================
@Invert
	LD HL,(DFile)		;DFile
	LD B,19 		;Row count
	DEC HL 
@ILoop
	INC HL 
	LD A,(HL) 
	CP 76 
	JR NZ NotNL1
	DJNZ ILoop
	LD C,00 
	RET
@NotNL1
	ADD A,80 
	LD (HL),A
	JR ILoop
;========================================
@DELAY
	LD BC,0900 
@DLoop
	DEC BC 
	LD A,B 
	OR C 
	JR NZ DLoop
	RET
;========================================
;Explosion FX
;========================================
@Explosion
	LD HL,(DFile) 		;DFile
	LD BC,017B		;Offset
	ADD HL,BC
	LD (HL),97		;Graph *
	LD A,0B 
	LD DE,0021 
	LD BC,0001 
@Boom
	PUSH HL			;Explosion FX
	ADD HL,BC
	LD (HL),06
	SBC HL,DE 
	LD (HL),06
	SBC HL,BC 
	LD (HL),03
	SBC HL,BC 
	LD (HL),86
	ADD HL,DE 
	LD (HL),86
	ADD HL,DE 
	LD (HL),06
	ADD HL,BC 
	LD (HL),83
	ADD HL,BC 
	LD (HL),86
	INC BC 
	LD HL,0021 
	ADD HL,DE 
	EX DE,HL 
	PUSH AF 
	PUSH BC 
	CALL DELAY
	POP BC 
	POP AF 
	POP HL 
	DEC A 
	JR NZ Boom
	RET
;========================================
;Draw the enterprise
;========================================
@Enterprise
	LD HL,(DFile)
	LD DE,00C7		;Offset
	ADD HL,DE		;Add offset to DFile
	EX DE,HL		;DE=LDIR destination
	LD HL,NCC17O1		;HL=LDIR source
	LD BC,00E7		;BC=LDIR count
	LDIR
	RET

@NCC17O1
	80 80 80 80 80 80 80 80 03 03 80 80 80 80 80 80 03 03 03 03 03 03 03 03 03 00 00 03 03 80 80 80 76
	80 80 80 03 03 03 03 00 09 09 00 03 03 03 03 05 00 33 28 28 00 1D 23 34 1D 00 09 00 81 80 80 80 76 
	80 80 80 82 00 1B 1B 00 09 00 16 16 1B 00 81 80 83 83 83 00 83 83 83 83 83 83 83 81 80 80 80 80 76 
	80 80 80 80 80 80 83 00 0E 1B 00 04 00 02 84 80 80 80 80 00 80 80 80 80 80 80 80 80 80 80 80 80 76 
	80 80 80 80 80 80 80 80 04 87 80 80 01 00 00 1B 1B 00 03 00 03 03 80 80 80 80 80 80 80 80 80 80 76 
	80 80 80 80 80 80 80 80 80 80 96 91 88 1B 10 17 11 1B 16 1B 00 81 80 80 80 80 80 80 80 80 80 80 76 
	80 80 80 80 80 80 80 80 80 80 80 80 04 00 15 00 1B 1B 16 00 81 80 80 80 80 80 80 80 80 80 80 80 76
